/***************************************************************************
                          localfs.h  -  description
                             -------------------
    begin                : mon oct 28 2002
    copyright            : (C) 2002,2003 by Piotr Mierzwiski
    email                : peterm@go2.pl

    Copyright: See COPYING file that comes with this project
***************************************************************************/

#ifndef _LOCALFS_H_
#define _LOCALFS_H_

#include "virtualfs.h"
#include "enums.h"
#include "solf.h"

/**
 * Author:
 * Piotr Mierzwiski
 **/
/** @short Klasa jest odpowiedzialna za obsug lokalnego systemu plikw.
 Uruchamiane s wszelkie operacje plikowe na tym systemie plikw, takie
 jak: listowanie katalogu, kopiowanie, przenoszenie, usuwanie, zmiana nazwy,
 tworzenie katalogu, tworzenie pustego pliku, wczytywanie pliku do podgldu,
 waenie katalogw, tworzenie linkw, ustawianie atrybutw, wyszukiwanie pliku.
 Poza tym zawarta jest tu obsuga zakoczenia i startowania poszczeglnych
 polece systemu plikw (min. obsuga bdw) oraz zakoczenia operacji.
 Klasa poredniczy pomidzy obiektem klasy VirtualFS a najniej operujcy
 obiektem klasy SoLF. Zawiera metody, ktre zwracaj podstawowe informacje
 o systemie plikw i przetwarzanych plikach.
*/
class LocalFS : public VirtualFS
{
	Q_OBJECT
public:
	/** Konstruktor klasy.\n
	 @param inputURL - nazwa biecej cieki,
	 @param listViewExt - wskanik na obiekt widoku listy plikw.\n
	 cieka w parametrze jest wykorzystywana przez klas bazow systemu plikw,
	 tj. klas VirtualFS.
	 */
	LocalFS( const QString & inputURL, ListViewExt *listViewExt );

	/** Destruktor klasy.\n
	 Usuwany jest tutaj dialog postpu operacji oraz obiekt klasy SoLF.
	 */
	~LocalFS();

	/** Zwraca kod bdu dla ostatniej operacji plikowej.
	 @return kod bdu, patrz: @see Error.
	 */
	Error errorCode() const { return mSoLF->errorCode(); }

	/** Zwraca nazw gwnego katalogu.\n
	 @return nazwa gwnego katalogu - "/".
	 */
	QString host()    const { return QString("/"); }

	/** Zwraca biec ciek absolutn.\n
	 @return bieca cieka absolutna.
	 */
	QString path()    const { return mCurrentPath; }

	/** Zwraca nazw (ze ciek absolut) aktualnie przetwarzanego pliku.
	 @return nazwa pliku ze cie absolutn.
	 */
	QString nameOfProcessedFile() const {
		return mSoLF->nameOfProcessedFile();
	}

	/** Wywoanie funkcji pobierajcej informacje o systemie plikw dla podanego
	 katalogu.\n
	 @param path - nazwa katalogu w systemie plikw,
	 @param freeBytes - zmienna, w ktrej naley umieci ilo wolnych bajtw,
	 @param allBytes - zmienna, w ktrej naley umieci ilo wszystkich bajtw.
	 @return TRUE, jeli udao si pobra informacje, w przeciwnym razie FALSE.
	 */
	bool getStatFS( const QString & path, long long & freeBytes, long long & allBytes ) {
		return SoLF::getStatFS( path, freeBytes, allBytes );
	}

	/** Uruchamia funkcj pobierajc wyniki operacji waenia, tj. waga caoci,
	 ilo plikw i katalogw.\n
	 @param weight - zmienna, w ktrej zapisana bdzie waga,
	 @param files - zmienna do zapisania iloci plikw,
	 @param dirs - zmienna do zapisania iloci katalogw.
	 */
	void getWeighingStat( long long & weigh, uint & files, uint & dirs ) {
		mSoLF->getWeighingStat( weigh, files, dirs );
	}

	/** Pobiera informacj o pliku wczeniej zainicjowan w skadowej
	 @em mNewUrlInfo .\n
	 @param newUrlInfo - zmienna, w ktrej naley umieci informacj o pliku.\n
	 Metoda jest wykorzystywana przez @em FilesPanel::slotResultOperation() ,
	 w celu zapewnienia odpowiedniej obsugi widoku listy.
	 */
	void getNewUrlInfo( UrlInfoExt & newUrlInfo ) {
		newUrlInfo = mNewUrlInfo;
	}

	/** Zwraca informacj o tym czy biecy katalog ma prawa zapisu dla aktualnie
	 zalogowanego uytkownika.\n
	 @return informacja o zapisie typu @em DirWritable
	 \n \n Patrz te: @see DirWritable.
	 */
	DirWritable currentDirWritable() {
		return QFileInfo(currentURL()).isWritable() ? ENABLEdw : DISABLEdw;
	}


	/** Funkcja zwraca nazw uytkownika lub grupy, aktualnie zalogowanego lub
	 o podanym numerze Id.\n
	 @param user - rwne TRUE (domylnie) wymusza zwrcenie nazwy uytkownika,
	  w przeciwnym razie jest to grupa,
	 @param id - mniejsze od zera (domylnie) wymusza zwrcenie nazwy zalogowanego
	  uytkownika bd grupy, do ktrej naley. Wiksze od zera nakazuje zwrcenie
	  nazwy uytkownika lub grupy podanego Id,
	 @return nazwa uytkownika lub grupy.
	 */
	static QString getUser( bool user=TRUE, int id=-1 );

	/** Funkcja umoliwia pobranie listy uytkownikw lub grup, albo numeru Id
	 podanego uytkownika lub grupy.
	 @param getUser - rwne TRUE wymusza pobranie uytkownika(w), w przeciwnym
	  razie pobierane s grupy,
	 @param list - adres listy na ktr wstawiani s pobrani uytkownicy bd grupy;
	  podawana jest te przez ten parametr nazwa uytkownika lub grupy, ktrej Id
	  powinno by zwrcone,
	 @param getAll - rwne TRUE wymusza zapisanie na list wszystkich dostpnych
	  uytkownikw lub grup w systemie, w przeciwnym razie na list wstawiane s
	  tylko nazwy grup dla aktualnie zalogowanego,
	 @param id - numer Id uytkownika lub grupy, ktrej nazw naley zapisa na
	  licie,
	 @param returnId - rwne TRUE, nakazuje zwrcenie numeru Id uytkownika lub
	  grupy podanej na licie, w przciwnym razie zwracana jest warto -1,
	 @return numer Id uytkownika lub grupy.
	*/
	static int getOwners( bool getUser, QStringList & list, bool getAll, int id=-1, bool returnId=FALSE );

	/** Funkcja zwraca status zachowania fragmentu kopiowanego lub przenoszonego pliku.
	@return TRUE jezeli zachowano czesc pliku, w przeciwnym razie FALSE.
	*/
	bool afterBreakKeepFragment() { return mSoLF->afterBreakKeepFragment(); }

private:
	SoLF *mSoLF;
	QString mCurrentPath;

	bool mOperationHasFinished, mSkipNextErrorOccures;
	uint mOpFileCounter, mTotalFiles;
	UrlInfoExt mNewUrlInfo;

	QFile mFile;
	bool mFileHasBeenOpened;

	bool mRemoveToTrash, mWeighBeforeOperation;
	QString mTrashDirectory;

protected:
	/** Metoda suca do otwierania podanego pliku do podgldu.\n
	 @param fileName - nazwa pliku do otwarcia.\n
	 Plik w rzeczywistoci nie jest otwierany, sprawdzane jest tylko, czy on
	 istnieje i czy mona go odczytywa, jeli to prawda, wtedy wywoywana jest
	 procedura uruchamiajca podgld tego pliku.
	 */
	void open( const QString & fileName );

	/** Metoda sprawdza czy podany plik istnieje i czy mona go odczytywa.\n
	 @param fileName - nazwa pliku do sprawdzenia.
	 @param silent - TRUE spowoduje, e nie zostanie pokazana informacja o bdzie
	 @return TRUE jeli sprawdzanie zakoczyo si sukcesem, w przeciwnym
	  wypadku FALSE.
	 */
	bool fileIsReadable( const QString & fileName, bool silent=FALSE );

	/** Metoda uruchamia listowanie plikw z podanego katalogu.\n
	 @param fullPathName - cieka katalogu, ktry naley wylistowa.
	 */
	void readDir( const QString & fullPathName );

	/** Metoda uruchamia procedur tworzenia nowego katalogu lub struktury
	 katalogw o podanej nazwie.\n
	 @param dirName - nazwa katalogu lub struktury do utworzenia.
	 */
	void mkdir( const QString & dirName );

	/** Metoda uruchamia procedur utworzenia nowego pliku lub katalogu.\n
	 @param fullFileName - nazwa pliku lub katalogu do utworzenia.\n
	 Jeli nazwa koczy sie znakiem '/', wtedy wywoywane jest polecenie
	 tworzenia nowego katalogu, w przeciwnym razie tworzony jest pusty plik.
	*/
	void createAnEmptyFile( const QString & fullFileName );

	/** Uruchamia kopiowanie lub przenoszenie plikw z podanej listy do podanego
	 katalogu.\n
	 @param targetPath - cieka docelowa kopiowania lub przenoszenia,
	 @param filesList - lista plikw do skopiowania lub przeniesienia,
	 @param savePerm - rwne TRUE nakazuje zachowywanie atrybutw kopiowanych plikw,
	 @param alwaysOvr - jeli ma warto TRUE spowoduje, e pliki zawsze bd
	  nadpisywane,
	 @param followByLink - rwne TRUE spowoduje skopiowanie nie linkw, ale plikw
	  na ktry kady link wskazuje,
	 @param move - rwne TRUE wymusza przenoszenie plikw, w przeciwnym razie
	  nastpuje kopiowanie.
	 */
	void copyFiles( QString & targetPath, const QStringList & filesList, bool savePerm, bool alwaysOvr, bool followByLink, bool move );

	/** Metoda uruchamia usuwanie plikw z podanej listy.\n
	 @param filesList - lista plikw do usunicia,
	 */
	void deleteFiles( const QStringList & filesList );

	/** Metoda uruchamia waenie podanych na licie plikw.\n
	 @param filesList - lista plikw do zwaenia
	 @param realWeight - rwne TRUE spowoduje, e funkcja waca uwzgldni
	  rzeczywist wag pliku w systemie plikw.
	 */
	void sizeFiles( const QStringList & filesList, bool realWeight );

	/** Metoda uruchamia tworzenie linku lub grupy linkw, albo edycj
	 pojedynczego.\n
	 @param sourceFilesList - lista rdowych plikw,
	 @param targetFilesList - lista nazw docelowych linkw,
	 @param editOnly - rwny TRUE, mwi e nastpi tylko edycja jednego linku,
	 @param hard - rwny TRUE, wymusza tworzenie tzw.twartych linkw, w przeciwnym
	  razie tworzone s tzw. mikkie linki,
	 @param alwaysOvr - rwny TRUE, spowoduje e linki zawsze bd nadpisywane.
	 */
	void makeLink( const QStringList & sourceFilesList, const QStringList & targetFilesList, bool editOnly, bool hard, bool alwaysOvr );

	/** Metoda uruchamia ustawianie atrybutw dla podanych na licie plikw.\n
	 @param filesList - lista plikw, ktrym naley ustawi atrybuty,
	 @param newUrlInfo - informacja o atrybutach dla plikw,
	 @param recursive - rwne TRUE, spowoduje e atrybuty bd zmieniane dla caej
	  zawartoci katalogw z listy,
	 @param changesFor - rodzaj zmiany atrybutw, dozwolone s:
	  @li 0 - zmiana dla wszystkich,
	  @li 1 - zmiana tylko dla plikw,
	  @li 2 - zmiana tylko dla katalogw.
	 */
	void setAttributs( const QStringList & filesList, const UrlInfoExt & newUrlInfo, bool recursive, int changesFor );

	/** Metoda wywoywana po uruchomieniu wyszukiwania na dialogu szukania pliku.\n
	 @param findCriterion - adres obiektu zawierajcego kryteria wyszukiwania.\n
	 Uruchamiane jest tutaj wyszukiwanie pliku.
	 */
	void startFindFile( const FindCriterion & findCriterion );

	/** Zwraca kod biecej operacji, patrz: @see Operation.\n
	 @return kod biecej operacji.
	 */
	Operation currentOperation() const { return mSoLF->currentOperation(); }

	/** Funkcja uruchamia procedur przerwania biecej operacji.
	 */
	void breakCurrentOperation();

	/** Zwraca informacje o tym czy podany plik jest linkiem symbolicznym.\n
	 @return TRUE podany plik jest linkiem, FALSE - plik nie jest linkiem.
	 */
	bool isLink( const QString & fileName ) { return QFileInfo( fileName ).isSymLink(); }

	/** Zwraca ciek docelow dla podanego linku.\n
	 @return cieka docelowa linku.
	 */
	QString readLink( const QString & fileName ) { return QFileInfo( fileName ).readLink(); }

	/** Zwraca nazw aktualnie zalogowanego uytkownika.\n
	 @return nazwa uytkownika.
	 */
	QString loggedOwner() { return getUser(); }

	/** Zwraca nazw grupy dla aktualnie zalogowanego uytkownika.\n
	 @return nazwa grupy.
	 */
	QString loggedGroup() { return getUser( FALSE ); }

	/** Zwraca liczb wszystkich plikw i katalogw w biecym katalogu.\n
	 @return suma plikw i katalogw z biecego.
	 */
	uint numOfAllFiles() { return mSoLF->numOfAllFiles(); }

	/** Metoda wykorzystywana do inicjowania skladowej klasy @em mNewUrlInfo .\n
	 @param fileName - nazwa pliku, ktremu naley pobra atrybuty.\n
	 \n Patrz te: @em getNewUrlInfo()
	 */
	void initNewUrlInfo( const QString & fileName=QString::null );

	/** Wcza bd wycza bezpieczne usuwnie plikw.\n
	 @param toTrash - TRUE wcza bezpieczne usuwanie (przenoszenie do "kosza"),
	  FALSE wcza rzeczywiste usuwanie (domylne ustawianie).\n
	 Patrz te: @see setTrashPath().
	 */
	void setRemoveToTrash( bool toTrash ) { mRemoveToTrash = toTrash; }

	/** Ustawia ciek dla katalogu "kosza".\n
	 @param path - cieka do katalogu "kosza".
	 */
	void setTrashPath( const QString & path ) { mTrashDirectory = path; }

	/** Wcza albo wycza czynno waenia plikw i katalogw przed operacj
	 plikow (np. kopiowania).\n
	 @param weighBefore - warto TRUE spowoduje, e pliki i katalogi bd waone,
	  w przeciwnym razie czynno ta zostanie pominita (domylnie jest to
	  wczone, wyczenie przypiesza operacj plikow, ale nie jest dostpna
	  informacja o jej  postpie).
	 */
	void setWeighBeforeOperation( bool weighBefore ) { mWeighBeforeOperation = weighBefore; }

public slots:
	/** Slot uruchamia procedur wstrzymania lub wznowienia biecej operacji.\n
	 @param stop - rwne TRUE, wymusza wstrzymanie operacji, FALSE powoduje
	  wznowienie wstrzymanej.
	 */
	void slotPause( bool stop );

	/** Slot powoduje zainicjowanie podanego obiektu z informacj o pliku
	 atrybutami podanego.\n
	 @param fullFileName - nazwa pliku, ktremu naley pobra atrybuty,
	 @param urlInfoExt - adres obiektu, ktry naley zainicjowa atrybutami.\n
	 Przy czym pobierane s tylko nastpujce atrybuty: nazwa, rozmiar oraz czas
	 ostatniej modyfikacji podanego pliku.
	 */
	void slotGetUrlInfo( const QString & fullFileName, UrlInfoExt & urlInfoExt );

	/** Slot powoduje wczytywanie pliku wybranego do podgldu.\n
	 @param fileName - nazwa pliku do wczytania,
	 @param buffer - adres bufora, w ktrym naley umieszcza wczytywane dane,
	 @param bytesRead - ilo wczytywanych na raz bajtw, a po odczycie zawiera
	  ilo faktycznie wczytanych bajtw (0 oznacza koniec pliku),
	 @param fileLoadMethod - metoda wczytywania pliku, patrz @see View::UpdateMode
	 */
	void slotReadFileToView( const QString & fileName, QByteArray & buffer, int & readBlockSize, int fileLoadMethod );

private slots:
	/** Slot wywoywany w momenecie zakonczenia operacji.\n
	 @param error - rwne TRUE, mwi e wystpi bd.\n
	 Operacja zostaa zakoczona jeli wykonano ostatnie polecenie z kolejki, lub
	 wystpi bd "nie do ominicia". Przykadow operacj moe by przenoszenie,
	 a ostatnim jego poleceniem jest usuwanie.
	 */
	void slotDone( bool error );

	/** Slot powoduje wywietlenie info o biecej operacji na pasku statusu.\n
	 @param state - kod operacji (typu Operation).\n
	 Wysyany jest tutaj tylko sygna @em signalSetOperation() .
	 */
	void slotStateChanged( int state );

	/** Slot wywoywany w momencie rozpoczcia wykonywania polecenia.\n
	 @param cmd - kod polecenia (typu Operation).
	 */
	void slotCommandStarted( int command );

	/** Slot wywoywany w chwili zakoczenia wykonywania podanego polecenia.\n
	 @param cmd - kod polecenia ( typu @em Operation ),
	 @param error - rwne TRUE oznacza, e pojawi si bd w wykonanym poleceniu.\n
	 Jeli prba wykonania polecenia zakoczya si bdem pokazywany jest
	 komunikat z informacj o bdzie i nazwy pliku lub katalogu, ktrego bd
	 dotyczny.
	 */
	void slotCommandFinished( int command, bool error );


	/** Slot wywoywany przez obiekt klasy FileView, po wykonaniu zapisu do nowego
	 lub ju otwartego pliku.\n
	 @param fileName - nazwa pliku, do ktrego nastpuje zapis.
	 */
	void slotUpdateItemOnLV( const QString & fileName );

};

#endif
